/*
Copyright (c) 2008 Yahoo! Inc.  All rights reserved.  
The copyrights embodied in the content of this file are licensed under the BSD (revised) open source license
*/
package
{
	import com.yahoo.webapis.answers.Question;
	
	import flash.events.MouseEvent;
	import flash.net.URLRequest;
	import flash.net.navigateToURL;
	
	import mx.controls.Button;
	import mx.controls.LinkButton;
	import mx.controls.ProgressBar;
	import mx.core.UIComponent;
	import mx.core.UITextField;


	/**
	 * Displayed a paged view of questions asked on Yahoo! Answers.
	 */
	public class AnswersViewer extends UIComponent
	{
		
	//--------------------------------------
	//  Class Constants
	//--------------------------------------
	
		[Embed(source="assets/arrows.swf", symbol="LeftArrow")]
		private static const LEFT_ARROW:Class;
	
		[Embed(source="assets/arrows.swf", symbol="RightArrow")]
		private static const RIGHT_ARROW:Class;
	
		private static const BUTTON_SIZE:Number = 20;
		private static const SPACING:Number = 8;
		
	//--------------------------------------
	//  Constructor
	//--------------------------------------
	
		/**
		 * Constructor.
		 */
		public function AnswersViewer()
		{
			super();
		}
		
	//--------------------------------------
	//  Properties
	//--------------------------------------
		
		/**
		 * @private
		 * The progress bar displayed while the data provider is empty.
		 */
		protected var progressBar:ProgressBar;
		
		/**
		 * @private
		 * The button to press to show the next question.
		 */
		protected var nextButton:Button;
		
		/**
		 * @private
		 * The button to press to show the previous question.
		 */
		protected var previousButton:Button;
		
		/**
		 * @private
		 * Displays the subject of the question.
		 */
		protected var subject:UITextField;
		
		/**
		 * @private
		 * Displays the author of the question.
		 */
		protected var author:UITextField;
		
		/**
		 * @private
		 * Displays the number of answers to the question and links to the page on Y! Answers.
		 */
		protected var link:LinkButton;
		
		/**
		 * @private
		 * Storage for the selectedIndex property.
		 */
		private var _selectedIndex:int = 0;
		
		/**
		 * The index of the currently visible item.
		 */
		public function get selectedIndex():int
		{
			return this._selectedIndex;
		}
		
		/**
		 * @private
		 */
		public function set selectedIndex(value:int):void
		{
			if(this._selectedIndex != value)
			{
				this._selectedIndex = value;
				this.invalidateProperties();
				this.invalidateSize();
				this.invalidateDisplayList();
			}
		}
		
		/**
		 * @private
		 * Storage for the dataProvider property.
		 */
		private var _dataProvider:Array;
		
		/**
		 * The data from Yahoo! Answers that gets displayed in this control.
		 * Should be an Array containing Question objects.
		 * 
		 * @see com.yahoo.webapis.answers.Question
		 */
		public function get dataProvider():Array
		{
			return this._dataProvider;
		}
		
		/**
		 * @private
		 */
		public function set dataProvider(value:Array):void
		{
			this._dataProvider = value;
			this._selectedIndex = 0;
			this.invalidateProperties();
			this.invalidateSize();
			this.invalidateDisplayList();
		}
		
	//--------------------------------------
	//  Protected Methods
	//--------------------------------------
		
		/**
		 * @private
		 */
		override protected function createChildren():void
		{
			super.createChildren();
			
			if(!this.progressBar)
			{
				this.progressBar = new ProgressBar();
				this.progressBar.indeterminate = true;
				this.progressBar.label = "Loading Yahoo! Answers Data";
				this.addChild(this.progressBar);
			}
			
			if(!this.previousButton)
			{
				this.previousButton = new Button();
				this.previousButton.setStyle("icon", LEFT_ARROW);
				this.addChild(this.previousButton);
			}
			this.previousButton.addEventListener(MouseEvent.CLICK, previousClickHandler);
			
			if(!this.nextButton)
			{
				this.nextButton = new Button();
				this.nextButton.setStyle("icon", RIGHT_ARROW);
				this.addChild(this.nextButton);
			}
			this.nextButton.addEventListener(MouseEvent.CLICK, nextClickHandler);
			
			if(!this.subject)
			{
				this.subject = new UITextField();
				this.subject.styleName = this;
				this.subject.multiline = true;
				this.subject.wordWrap = true;
				this.subject.selectable = true;
				this.addChild(this.subject);
			}
			
			if(!this.author)
			{
				this.author = new UITextField();
				this.author.styleName = this;
				this.addChild(this.author);
			}
			
			if(!this.link)
			{
				this.link = new LinkButton();
				this.link.toolTip = "Visit Yahoo! Answers";
				this.link.addEventListener(MouseEvent.CLICK, linkClickHandler);
				this.addChild(this.link);
			}
		}
		
		/**
		 * @private
		 */
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if(this.dataProvider)
			{
				if(this.dataProvider.length > 0)
				{
					if(this.selectedIndex < 0 || this.selectedIndex >= this.dataProvider.length)
						this._selectedIndex = 0;
						
					var currentQuestion:Question = this.dataProvider[this.selectedIndex];
					
					this.author.htmlText = '<font color="#46A209" size="14"><b>' + currentQuestion.author.userNick + '</b></font> asks:';
					
					this.subject.text = currentQuestion.subject;
					
					this.link.label = currentQuestion.numAnswers + " answers";
					
					this.nextButton.enabled = this.selectedIndex < this.dataProvider.length - 1;
					this.previousButton.enabled = this.selectedIndex > 0;
				}
				else
				{
					this.author.htmlText = '<font color="#46A209" size="14"><b>Yahoo! Answers</b></font>';
					this.subject.text = "No questions found.";
					this.link.label = "";
					this.nextButton.enabled = this.previousButton.enabled = false;
				}
			}
			else
			{
				this.subject.text = "";
				this.author.text = "";
				this.link.label = "";
			}
			
			this.progressBar.visible = this.dataProvider == null;
			this.nextButton.visible = this.previousButton.visible = !progressBar.visible;
			
		}
		
		/**
		 * @private
		 */
		override protected function measure():void
		{
			super.measure();
			
			if(!this.dataProvider || this.dataProvider.length == 0) //we only have a progress bar
			{
				if(isNaN(this.explicitWidth)) this.measuredWidth = 100;
				else this.measuredWidth = this.explicitWidth;
				
				if(isNaN(this.explicitHeight)) this.measuredHeight = 50;
				else this.measuredHeight = this.explicitHeight;
			}
			else
			{
				if(isNaN(this.explicitWidth))
				{
					this.measuredWidth = Math.max(this.measuredWidth, this.author.measuredWidth);
					this.measuredWidth = Math.max(this.measuredWidth, this.subject.measuredWidth);
					this.measuredWidth = Math.max(this.measuredWidth, this.link.measuredWidth);
					this.measuredWidth += 2 * BUTTON_SIZE + SPACING;
				}
				else
				{
					this.measuredWidth = this.explicitWidth;
					this.subject.setActualSize(Math.max(this.explicitWidth - 2 * BUTTON_SIZE - SPACING, 4), this.subject.height);
				}
				
				if(isNaN(this.explicitHeight))
				{
					this.measuredHeight = this.subject.getExplicitOrMeasuredHeight();
					this.measuredHeight += this.author.getExplicitOrMeasuredHeight();
					this.measuredHeight += this.link.getExplicitOrMeasuredHeight();
				}
				else this.measuredHeight = this.explicitHeight;
			}
		}
		
		/**
		 * @private
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			this.progressBar.setActualSize(unscaledWidth, unscaledHeight);
			
			var xPosition:Number = 0;
			
			//position the buttons first to determine the content size
			this.nextButton.x = unscaledWidth - BUTTON_SIZE;
			this.nextButton.setActualSize(BUTTON_SIZE, BUTTON_SIZE);
			this.nextButton.y = (unscaledHeight - BUTTON_SIZE) / 2;
			
			this.previousButton.x = xPosition;
			this.previousButton.setActualSize(BUTTON_SIZE, BUTTON_SIZE);
			this.previousButton.y = (unscaledHeight - BUTTON_SIZE) / 2;
			xPosition += this.previousButton.width;
			
			var dataWidth:Number = this.nextButton.x - this.previousButton.width - SPACING;
			
			//spacing between the previous button and the viewable content.
			xPosition += SPACING / 2;
			
			var yPosition:Number = 0;
			
			this.author.x = xPosition;
			this.author.y = yPosition;
			this.author.setActualSize(dataWidth, this.author.getExplicitOrMeasuredHeight());
			yPosition += this.author.height;
			
			this.subject.x = xPosition;
			this.subject.y = yPosition;
			this.subject.setActualSize(dataWidth, this.subject.getExplicitOrMeasuredHeight());
			yPosition += this.subject.height;
			
			this.link.x = xPosition - 8; //8 pixels are added to the LinkButton for spacing, but it's not a style!
			this.link.y = yPosition;
			this.link.setActualSize(this.link.measuredWidth, this.link.measuredHeight);
			
		}
		
	//--------------------------------------
	//  Private Methods
	//--------------------------------------
	
		/**
		 * @private
		 * Shows the next question.
		 */
		private function nextClickHandler(event:MouseEvent):void
		{
			//doesn't check if the next question exists because we
			//disable the next button when we reach the end.
			this.selectedIndex++;
		}
		
		/**
		 * @private
		 * Shows the previous question.
		 */
		private function previousClickHandler(event:MouseEvent):void
		{
			//doesn't check if the previous question exists because we
			//disable the previous button when we reach the end.
			this.selectedIndex--;
		}
		
		/**
		 * @private
		 * Redirects to the question page on Y! Answers.
		 */
		private function linkClickHandler(event:MouseEvent):void
		{
			if(this.dataProvider.length > 0)
			{
				var question:Question = this.dataProvider[this.selectedIndex];
				navigateToURL(new URLRequest(question.link), "_self");
			}
		}
		
	}
}