package com.winnb.locator.services
{
	import com.winnb.locator.data.LocationVO; 
	import com.winnb.locator.data.MarkerVO; 
	import com.winnb.locator.data.QueryVO; 
	import com.winnb.locator.events.DBResultEvent; 
	import flash.data.SQLConnection; 
	import flash.data.SQLResult; 
	import flash.data.SQLStatement; 
	import flash.events.Event; 
	import flash.events.EventDispatcher; 
	import flash.events.SQLErrorEvent; 
	import flash.events.SQLEvent; 
	import flash.filesystem.File; 
	[Event(name="creationComplete", type="flash.events.Event")] 
	[Event(name="creationError", type="flash.events.Event")] 

	final public class DBService extends EventDispatcher 
	{
		private var _connection:SQLConnection; 
		private var _statement:SQLStatement; 
		private var _dbFile:File; 
		private var _isOpen:Boolean; 
		private var _queryQueue:Array; 
		private var _currentQuery:QueryVO; 
		private var _isRunningQuery:Boolean; 
		private static const FALL_THROUGH:String = ""; 
		private static const CREATE_TABLE_LOC:String = "createLocations";
		private static const CREATE_TABLE_MARK:String = "createMarkers";
		public static const CREATION_COMPLETE:String = "creationComplete"; 
		public static const CREATION_ERROR:String = "creationError"; 
		private static const DB:String = "Locator.db"; 

		public function DBService() 
		{ 
			_connection = new SQLConnection(); 
			_connection.addEventListener( SQLEvent.OPEN, onConnectionOpen ); 
			_connection.addEventListener( SQLEvent.CLOSE, onConnectionClose ); 
			_connection.addEventListener( SQLErrorEvent.ERROR, onConnectionError ); 
			_queryQueue = new Array(); 
			_dbFile = File.applicationStorageDirectory.resolvePath( DB ); 
			invalidate(); 
		}
		
		private function invalidate():void 
		{
			_statement = new SQLStatement(); 
			_statement.sqlConnection = _connection; 
			_statement.addEventListener(SQLEvent.RESULT, onCreationQueryResult); 
			_statement.addEventListener(SQLErrorEvent.ERROR, onCreationQueryError); 
			var sqlCreateLoc:String = "CREATE TABLE IF NOT EXISTS locations (" + 
									  "id INTEGER PRIMARY KEY," + 
									  "name TEXT NOT NULL," + 
									  "content TEXT NOT NULL);";
			var sqlCreateMark:String = "CREATE TABLE IF NOT EXISTS markers (" + 
									   "id INTEGER PRIMARY KEY," + 
									   "jsId TEXT NOT NULL," + 
									   "locationId TEXT NOT NULL," + 
									   "caption TEXT NOT NULL," + 
									   "x FLOAT NOT NULL," + 
									   "y FLOAT NOT NULL);"; 
			addQuery( new QueryVO( CREATE_TABLE_LOC, sqlCreateLoc ), false ); 
			addQuery( new QueryVO( CREATE_TABLE_MARK, sqlCreateMark ), true ); 
		}

		private function open():void 
		{
			if( _isOpen ) return; 
			_connection.open( _dbFile ); 
		}

		private function executeNextQuery():void 
		{
			if( !_isOpen ) open(); 
			else 
			{ 
				if( _queryQueue.length > 0 ) 
				{
					if( !_isRunningQuery ) _isRunningQuery = true; 
					_currentQuery = _queryQueue.shift(); 
					_statement.text = _currentQuery.query; 
					_statement.itemClass = _currentQuery.itemClass; 
					_statement.execute(); 
				} else {
					_isRunningQuery = false; 
					_currentQuery = null; 
					_connection.close(); 
				}
			}
		} 

		private function onConnectionOpen( evt:SQLEvent ):void 
		{ 
			_isOpen = true; 
			if( _queryQueue.length > 0 ) executeNextQuery(); 
		} 

		private function onConnectionClose( evt:SQLEvent ):void 
		{ 
			_isOpen = false; 
			if( _queryQueue.length > 0 ) 
			{ 
				_queryQueue.splice( 0, 0, _currentQuery ); 
				_currentQuery = null; 
				_isRunningQuery = false; 
			} 
		} 

		private function onConnectionError( evt:SQLErrorEvent ):void 
		{ 
			trace( "Database connection error!" ); 
		} 

		private function onCreationQueryResult( evt:SQLEvent ):void 
		{ 
			if( _queryQueue.length > 0 ) 
			{ 
				executeNextQuery(); 
			} else {
				_statement = new SQLStatement(); 
				_statement.sqlConnection = _connection; 
				_statement.addEventListener( SQLEvent.RESULT, onQueryResult ); 
				_statement.addEventListener( SQLErrorEvent.ERROR, onQueryError ); 
				dispatchEvent( new Event( CREATION_COMPLETE ) ); 
				_isRunningQuery = false; 
			} 
		} 

		private function onCreationQueryError( evt:SQLErrorEvent ):void 
		{ 
			dispatchEvent( new Event( CREATION_ERROR ) ); 
			_isRunningQuery = false; 
		} 

		private function onQueryResult( evt:SQLEvent ):void 
		{ 
			var result:SQLResult = _statement.getResult(); 
			if( _currentQuery.type != FALL_THROUGH ) 
				dispatchEvent( new DBResultEvent( _currentQuery.type, result ) ); 
			executeNextQuery(); 
		}

		private function onQueryError( evt:SQLErrorEvent ):void 
		{ 
			dispatchEvent( evt ); 
			_isRunningQuery = false; 
		} 

		public function saveLocation( location:LocationVO ):void 
		{ 
			location.serialize(); 
			var sqlSave:String; 
			var sqlSelect:String; 
			if( location.id == -1 ) 
			{ 
				// add new location... 
				sqlSave = "INSERT INTO locations VALUES (" + 
						  "null," + 
						  "’" + location.name + "’," + 
						  "’" + location.content + "’);"; 
				sqlSelect = "SELECT * FROM locations ORDER BY id DESC LIMIT 1;"; 
			} else {
				// update saved location... 
				sqlSave = "UPDATE locations SET " +
						  "name=’" + location.name + "’," +
						  "content=’" + location.content + "’ " +
						  "WHERE id=’" + location.id + "’;";
				sqlSelect = "SELECT * FROM locations WHERE id=’" + location.id + "’;"; 
			} 
			addQuery( new QueryVO( FALL_THROUGH, sqlSave ), false ); 
			addQuery( new QueryVO(DBResultEvent.LOCATION_SAVED, sqlSelect), true ); 
			location.deserialize();
		}

		public function saveMarker( marker:MarkerVO ):void
		{
			marker.serialize();
			var sqlSave:String;
			var sqlSelect:String;
			if( marker.id == -1 )
			{
				// add new marker...
				sqlSave = "INSERT INTO markers VALUES (" + 
						  "null," + 
						  "’" + marker.jsId + "’," + 
						  "’" + marker.locationId + "’," + 
						  "’" + marker.caption + "’," + 
						  "’" + marker.x + "’," +
						  "’" + marker.y + "’);";
				sqlSelect = "SELECT * FROM markers ORDER BY id DESC LIMIT 1;";
			} else {
				// update saved marker... 
				sqlSave = "UPDATE markers SET " + 
						  "locationId=’" + marker.locationId + "’," + 
						  "caption=’" + marker.caption + "’," + 
						  "x=’" + marker.x + "’," + 
						  "y=’" + marker.y + "’ " + 
						  "WHERE id=’" + marker.id + "’;"; 
				sqlSelect = "SELECT * FROM markers WHERE id=’" + marker.id + "’;"; 
			}
			addQuery( new QueryVO( FALL_THROUGH, sqlSave ), false );
			addQuery( new QueryVO( DBResultEvent.MARKER_SAVED, sqlSelect ),true );
			marker.deserialize(); 
		}

		public function retrieveLocation( location:String ):void
		{
			var sqlSelect:String = "SELECT * FROM locations WHERE " + "name=’" + escape( location ) + "’ " + "LIMIT 1;";
			addQuery( new QueryVO( DBResultEvent.LOCATION_RETRIEVED, sqlSelect, LocationVO ) ); 
		}

		public function retrieveAllLocations():void
		{
			var sqlSelect:String = "SELECT DISTINCT name FROM locations;";
			addQuery( new QueryVO(DBResultEvent.LOCATION_RETRIEVED_ALL, sqlSelect, LocationVO ) );
		}

		public function retrieveMarkersFromLocation( id:int ):void 
		{ 
			var sqlSelect:String = "SELECT * FROM markers WHERE " + "locationId=’" + id + "’;" 
			addQuery( new QueryVO( DBResultEvent.MARKER_RETRIEVED_ALL, sqlSelect, MarkerVO ) ); 
		}

		public function addQuery( query:QueryVO, run:Boolean = true ):void
		{
			_queryQueue.push( query );
			if( !_isRunningQuery && run ) runQuery();
		}

		public function runQuery():void
		{
			if( _queryQueue.length > 0 ) executeNextQuery();
		}

		public function shutDown():void 
		{
			if( _isRunningQuery )
			{
				_statement.removeEventListener(SQLEvent.RESULT, onQueryResult);
				_statement.removeEventListener(SQLErrorEvent.ERROR, onQueryError);
			}
			close();
		}

		public function close():void
		{
			if( _isOpen ) _connection.close();
		}
	}
}